//*******************************************************************************
//	Copyright (C) 2013 John Chapman

//	This software is distributed freely under the terms of the MIT License.
//	See "license.txt" or "http://copyfree.org/licenses/mit/license.txt".
//*******************************************************************************/

// Vertex vs
void PLF_vs
(
    in float4 inPos : POSITION
    ,out float4 pos : POSITION
    ,out float2 uv0 : TEXCOORD0
    ,uniform float4x4 worldViewProj
)
{
    pos = mul(worldViewProj, inPos);
    inPos.xy = sign(inPos.xy);
    uv0 = (float2(inPos.x, -inPos.y) + 1.0f) * 0.5f;
}

// Downsample ps
void PLFds_ps
(
    in float2 vTexcoord : TEXCOORD0
    ,out float4 fResult : COLOR
    ,uniform float uExp
	,uniform float uScale
	,uniform sampler2D uInputTex:register(s0)
)
{
	fResult = pow(tex2D(uInputTex, vTexcoord), float4(uExp,uExp,uExp,uExp)) * uScale;
}

// LensFlare ps
float4 textureDistortedHLSL(
	sampler2D tex, 
	float2 texcoord, 
	float2 direction,
	float3 distortion 
) {
	return float4(
		tex2D(tex, texcoord + direction * distortion.r).r,
		tex2D(tex, texcoord + direction * distortion.g).g,
		tex2D(tex, texcoord + direction * distortion.b).b,
		1.0
	);
}
//*----------------------------------------------------------------------------*/
void PLFlf_ps
(
    in float2 vTexcoord : TEXCOORD0
    ,out float4 fResult : COLOR
	,uniform sampler2D uInputTex:register(s0)
	,uniform sampler2D uLensColor:register(s1)
	,uniform int uSamples
	,uniform float uDispersal
	,uniform float uHaloWidth
	,uniform float uDistortion
	,uniform float4 uTextureSize
)
{
	float2 texcoord = -vTexcoord + float2(1.0,1.0); // flip texcoordoords
	float2 texelSize = 1.0 / float2(uTextureSize.xy);
	
	float2 ghostVec = (float2(0.5,0.5) - texcoord) * uDispersal;
	float2 haloVec = normalize(ghostVec) * uHaloWidth;
	
	float3 distortion = float3(-texelSize.x * uDistortion, 0.0, texelSize.x * uDistortion);

// sample ghosts:
	float4 result = float4(0.0,0.0,0.0,0.0);
	[loop]
	for (int i = 0; i < uSamples; ++i) {
		float2 offset = frac(texcoord + ghostVec * float(i));
		
		float weight = length(float2(0.5,0.5) - offset) / length(float2(0.5,0.5));
		weight = pow(1.0 - weight, 10.0);
		
		result += textureDistortedHLSL(
			uInputTex,
			offset,
			normalize(ghostVec),
			distortion
		) * weight;
	}

	result = result * tex2D(uLensColor, float2(length(float2(0.5,0.5) - texcoord) / length(float2(0.5,0.5)), 0.5));

//	sample halo:
	float weight = length(float2(0.5,0.5) - frac(texcoord + haloVec)) / length(float2(0.5,0.5));
	weight = pow(1.0 - weight, 10.0);
	result += textureDistortedHLSL(
		uInputTex,
		frac(texcoord + haloVec),
		normalize(ghostVec),
		distortion
	) * weight;
	
	fResult = result;
}	

// Blur
//*----------------------------------------------------------------------------*/
//*	Incremental, forward-differencing Gaussian elimination based on:
//	http://http.developer.nvidia.com/GPUGems3/gpugems3_ch40.html */

float4 incrementalGauss1DHLSL(
	sampler2D srcTex, 
	float2 srcTexelSize, 
	float2 origin,
	int radius,
	float2 direction
) {
	int nSamples = clamp(radius, 1, 128)/2;
	if (nSamples == 0)
		return tex2D(srcTex, origin);
	
	float SIGMA = float(radius) / 8.0;
	float sig2 = SIGMA * SIGMA;
	float TWO_PI	= 6.2831853071795;
		
//	set up incremental counter:
	float3 gaussInc;
	gaussInc.x = 1.0 / (sqrt(TWO_PI) * SIGMA);
	gaussInc.y = exp(-0.5 / sig2);
	gaussInc.z = gaussInc.y * gaussInc.y;
	
//	accumulate results:
	float4 result = tex2D(srcTex, origin) * gaussInc.x;	
	float loop = nSamples % 65;
	for (int i = 1; i < loop; ++i) {
		gaussInc.xy *= gaussInc.yz;
		
		float2 offset = float(i) * direction * srcTexelSize;
		result += tex2D(srcTex, origin - offset) * gaussInc.x;
		result += tex2D(srcTex, origin + offset) * gaussInc.x;
	}
	return result;
}
//*----------------------------------------------------------------------------*/
void PLFblr_ps
(
    in float2 vTexcoord : TEXCOORD0
    ,out float4 fResult : COLOR
	,uniform sampler2D uInputTex:register(s0)
//	,uniform int uBlurRadius
	,uniform float4 uBlurDirection
	,uniform float4 uTextureSizeBlr
)
{
	float2 texelSize = 1.0 / float2(uTextureSizeBlr.xy);
	fResult = incrementalGauss1DHLSL(uInputTex, texelSize, vTexcoord, 16/*uBlurRadius*/, uBlurDirection.xy); 
}

// Final
void PLFf_ps
(
    in float2 vTexcoord : TEXCOORD0
    ,out float4 fResult : COLOR
	,uniform sampler2D uInputTex:register(s0)
	,uniform sampler2D uLensFlareTex:register(s1)
	,uniform sampler2D uLensDirtTex:register(s2)
	,uniform sampler2D uLensStarTex:register(s3)
	,uniform float4x4 uLensStarMatrix
)
{
	float4 lensMod = tex2D(uLensDirtTex, vTexcoord);
	float2 lensStarTexcoord = mul(uLensStarMatrix , float4(vTexcoord, 1.0,1.0)).xy;
	lensMod += tex2D(uLensStarTex, lensStarTexcoord);

	float4 result = tex2D(uInputTex, vTexcoord);

	result += tex2D(uLensFlareTex, vTexcoord) * lensMod;

	fResult = result;
}	
