/*******************************************************************************
	Copyright (C) 2013 John Chapman

	This software is distributed freely under the terms of the MIT License.
	See "license.txt" or "http://copyfree.org/licenses/mit/license.txt".
*******************************************************************************/
#version 130

uniform sampler2D uInputTex;

uniform int uBlurRadius;
uniform vec4 uBlurDirection; // (1,0)/(0,1) for x/y pass
uniform vec4 uTextureSizeBlr;

in vec2 vTexcoord;
out vec4 fResult;

/*----------------------------------------------------------------------------*/
/*	Incremental, forward-differencing Gaussian elimination based on:
	http://http.developer.nvidia.com/GPUGems3/gpugems3_ch40.html */
vec4 incrementalGauss1D(
	in sampler2D srcTex, 
	in vec2 srcTexelSize, 
	in vec2 origin,
	in int radius,
	in vec2 direction
) {

	int nSamples = clamp(radius, 1, 128) / 2;
	
	if (nSamples == 0)
		return texture(srcTex, origin);
	
	float SIGMA = float(radius) / 8.0;
	float sig2 = SIGMA * SIGMA;
	float TWO_PI	= 6.2831853071795;
		
//	set up incremental counter:
	vec3 gaussInc;
	gaussInc.x = 1.0 / (sqrt(TWO_PI) * SIGMA);
	gaussInc.y = exp(-0.5 / sig2);
	gaussInc.z = gaussInc.y * gaussInc.y;
	
//	accumulate results:
	vec4 result = texture(srcTex, origin) * gaussInc.x;	
	for (int i = 1; i < nSamples; ++i) {
		gaussInc.xy *= gaussInc.yz;
		
		vec2 offset = float(i) * direction * srcTexelSize;
		result += texture(srcTex, origin - offset) * gaussInc.x;
		result += texture(srcTex, origin + offset) * gaussInc.x;
	}
	
	return result;
}

/*----------------------------------------------------------------------------*/
void main() {
	vec2 texelSize = 1.0 / vec2(uTextureSizeBlr.xy);
	fResult = incrementalGauss1D(uInputTex, texelSize, vTexcoord, uBlurRadius, uBlurDirection.xy); 
}